import { SliceImage, SpecialTag, Track } from "@/types";
import { useMarkStore } from "@/store";

export default function useMakeTrack() {
  const markStore = useMarkStore();

  // 标记分数轨迹
  function makeScoreTrack(
    event: MouseEvent,
    item: SliceImage,
    maxSliceWidth: number,
    theFinalHeight: number
  ) {
    if (
      !markStore.currentQuestion ||
      typeof markStore.currentScore === "undefined"
    )
      return;

    const target = event.target as HTMLImageElement;
    const track: Track = {
      mainNumber: markStore.currentQuestion?.mainNumber,
      subNumber: markStore.currentQuestion?.subNumber,
      score: markStore.currentScore,
      unanswered: Object.is(markStore.currentScore, -0),
      offsetIndex: item.indexInSliceUrls,
      // 新增轨迹时会校验轨迹是否距离边缘过近，使用round产生的误差可以忽略，不会发生超出边界的问题
      offsetX: Math.round(
        event.offsetX * (target.naturalWidth / target.width) + item.dx
      ),
      offsetY: Math.round(
        event.offsetY * (target.naturalHeight / target.height) + item.dy
      ),
      positionX: -1,
      positionY: -1,
      number: -1,
    };
    track.positionX = (track.offsetX - item.dx) / maxSliceWidth;
    track.positionY =
      (track.offsetY - item.dy + item.accumTopHeight) / theFinalHeight;

    if (track.offsetX > item.effectiveWidth + item.dx) {
      console.log("不在有效宽度内，轨迹不生效");
      return;
    }
    if (
      item.markerTrackList.some((t) => {
        return (
          Math.pow(Math.abs(t.offsetX - track.offsetX), 2) +
            Math.pow(Math.abs(t.offsetY - track.offsetY), 2) <
          500
        );
      })
    ) {
      console.log("两个轨迹相距过近");
      return;
    }
    // 是否保留当前的轨迹分
    const questionScore =
      markStore.currentTask &&
      markStore.currentQuestion &&
      markStore.currentTask.markResult.scoreList[
        markStore.currentQuestion.__index
      ];
    const ifKeepScore =
      Math.round(
        markStore.currentQuestion.maxScore * 1000 -
          (questionScore || 0) * 1000 -
          markStore.currentScore * 2 * 1000
      ) / 1000;
    if (ifKeepScore < 0 && markStore.currentScore > 0) {
      markStore.currentScore = undefined;
    }
    const markResult = markStore.currentTaskEnsured.markResult;
    const maxNumber =
      markResult.markerTrackList.length === 0
        ? 0
        : Math.max(...markResult.markerTrackList.map((t) => t.number));
    track.number = maxNumber + 1;

    markResult.markerTrackList = [...markResult.markerTrackList, track];
    const { __index, mainNumber, subNumber } = markStore.currentQuestion;
    markResult.scoreList[__index] =
      markResult.markerTrackList
        .filter((t) => t.mainNumber === mainNumber && t.subNumber === subNumber)
        .map((t) => t.score)
        .reduce((acc, v) => (acc += Math.round(v * 1000)), 0) / 1000;
    item.markerTrackList.push(track);
  }

  // 标记特殊轨迹 （特殊轨迹包括：文字、线条、圆圈）
  function makeSpecialTagTrack(
    event: MouseEvent,
    item: SliceImage,
    maxSliceWidth: number,
    theFinalHeight: number
  ) {
    if (
      !markStore.currentTask ||
      typeof markStore.currentSpecialTag === "undefined"
    )
      return;

    const target = event.target as HTMLImageElement;
    const track: SpecialTag = {
      mainNumber: markStore.currentQuestion?.mainNumber,
      subNumber: markStore.currentQuestion?.subNumber,
      tagName: markStore.currentSpecialTag,
      tagType: markStore.currentSpecialTagType,
      offsetIndex: item.indexInSliceUrls,
      offsetX: Math.round(
        event.offsetX * (target.naturalWidth / target.width) + item.dx
      ),
      offsetY: Math.round(
        event.offsetY * (target.naturalHeight / target.height) + item.dy
      ),
      positionX: -1,
      positionY: -1,
    };
    track.positionX = (track.offsetX - item.dx) / maxSliceWidth;
    track.positionY =
      (track.offsetY - item.dy + item.accumTopHeight) / theFinalHeight;

    if (
      item.tagList.some((t) => {
        return (
          Math.pow(Math.abs(t.offsetX - track.offsetX), 2) +
            Math.pow(Math.abs(t.offsetY - track.offsetY), 2) <
          500
        );
      })
    ) {
      console.log("两个轨迹相距过近");
      return;
    }
    markStore.currentTaskEnsured.markResult.markerTagList.push(track);
    item.tagList.push(track);
  }

  // 标记轨迹
  function makeTrack(
    event: MouseEvent,
    item: SliceImage,
    maxSliceWidth: number,
    theFinalHeight: number
  ) {
    if (markStore.currentSpecialTagType) {
      makeSpecialTagTrack(event, item, maxSliceWidth, theFinalHeight);
      if (markStore.currentSpecialTagType === "TEXT") {
        markStore.currentSpecialTag = undefined;
        markStore.currentSpecialTagType = undefined;
      }
    } else {
      makeScoreTrack(event, item, maxSliceWidth, theFinalHeight);
    }
  }

  return {
    makeScoreTrack,
    makeSpecialTagTrack,
    makeTrack,
  };
}
